import pandas
import numpy as np
import scipy.spatial
import chempy
import matplotlib as mpl
import matplotlib.pyplot as plt
import mplcursors

###
# ELEMENTOS
###

A, B, C = 'N', 'Lu' , 'H'
NA, NB, NC = 7, 71, 1

###
# READING DAT FILE
###

data = pandas.read_table('ternary_2p0.dat', usecols=[0, 5, 6, 9], names=['ID', 'distance', 'energy', 'formula'], delimiter='\s+', comment='#')

data = data.loc[data['energy']<=0]
data = data.loc[data['distance']<=0.15]

IDs = data['ID'].values
distances = data['distance'].values * 1000
energies = data['energy'].values
formulas = data['formula']

print(energies)

###
# PARSING COMPOSITIONS
###

formulas = [chempy.Substance.from_formula(f) for f in formulas]
stoichiometry = [f.composition for f in formulas]
composition = np.zeros((len(stoichiometry), 3))

for s in range(len(stoichiometry)):
    if NA in stoichiometry[s].keys():
        composition[s, 0] = stoichiometry[s][NA]
    if NB in stoichiometry[s].keys():
        composition[s, 1] = stoichiometry[s][NB]
    if NC in stoichiometry[s].keys():
        composition[s, 2] = stoichiometry[s][NC]

    ntot = composition[s].sum()
    composition[s] = composition[s] / ntot

xA, xB, xC = composition.T

###
# CONVERTING COMPOSITIONS TO GIBBS TRIANGLE
###

sin60, cos60 = .5 * np.sqrt(3), .5
def transform_coords(x1, x2):
    x1, x2 = np.array(x1), np.array(x2)
    x = x1 + x2 * cos60
    y = x2 * sin60
    return x, y

x, y = transform_coords(xB, xC)

###
# CREATING DATA ARRAY
###

points = np.column_stack((x, y, energies))

###
# FINDING CONVEX HULL
###

hull = scipy.spatial.ConvexHull(points)  # convex hull

#hullt = mtri.Triangulation(x[hull.vertices], y[hull.vertices])  # triangulating convex hull points

###
# CREATING GRAPHICS
###

fig = plt.figure(figsize=(8, 7))
ax = fig.add_subplot(aspect='equal')

mpl.rc('font',**{'family':'Palatino','serif':['Palatino'], 'size':10})
mpl.rc('text', usetex=True)

mpl.rcParams['axes.linewidth'] = 3.5
mpl.rcParams['xtick.major.width'] = 3.5
mpl.rcParams['ytick.major.width'] = 3.5

cmap='coolwarm'

# DRAWING GIBBS TRIANGLE

plt.axis('off')

plt.plot([0, 1, cos60, 0], [0, 0, sin60, 0], color='k')

ax.text(0, -.04, A, fontsize=20, ha='center', transform=ax.transAxes)
ax.text(1, -.04, B, fontsize=20, ha='center', transform=ax.transAxes)
ax.text(cos60, .12+sin60, C, fontsize=20, ha='center', transform=ax.transAxes)

ax.text(.5, -.05, f'$x_\mathrm{{{B}}}$', fontsize=18, ha='center', transform=ax.transAxes)
ax.text(.175, .53, f'$x_\mathrm{{{C}}}$', rotation=60, fontsize=18, ha='center', transform=ax.transAxes)

alpha = .1
for i in range(1, 10):
    ax.text(i/10, -.05, f'0.{i}', fontsize=12, ha='center')

    tickv = ax.plot([i/10,i/10], [0, -.01], color='k')

    xc, yc = transform_coords([0, -.012], [i/10, i/10])
    tickh = ax.plot(xc, yc, color='k')
    ax.text(xc[1]-.03, yc[1], f'0.{i}', fontsize=12, ha='center', va='center')

    xc, yc = transform_coords([0, 1-i/10], [i/10, i/10])
    line1 = ax.plot(xc, yc, color='k', alpha=alpha)
    
    xc, yc = transform_coords([i/10, i/10], [0, 1-i/10])
    line2 = ax.plot(xc, yc, color='k', alpha=alpha)

    xc, yc = transform_coords([i/10, 0], [0, i/10])
    line2 = ax.plot(xc, yc, color='k', alpha=alpha)

# PLOTTING DATA

# TRIANGULATION

for trio in hull.simplices:
    x1, x2 = points[trio][:, :2].T
    x1 = np.append(x1, x1[0])
    x2 = np.append(x2, x2[0])
    plt.plot(x1, x2, 'k-', zorder=3)

# DATA POINTS

allpoints = ax.scatter(x, y, c=distances, s=180, marker='s', edgecolor='black', linewidths=1, cmap=cmap, zorder=1, alpha=0.5)
hullpoints = ax.scatter(x[hull.vertices], y[hull.vertices], c=distances[hull.vertices], marker='.', edgecolor='black', cmap=cmap, linewidth=2,  s=1500, zorder=5, alpha=1)


###
# TOOLTIPS
###

def cursor_annotations(sel):
    sel.annotation.set_text(f'{IDs[sel.index]}\nDistance to hull: {distances[sel.index]:.1f} meV\nComposition: {formulas[sel.index]}')
    sel.annotation.get_bbox_patch().set(fc="lightsalmon", alpha=0.9)

crs = mplcursors.cursor(allpoints, hover=True)
crs.connect("add", cursor_annotations)

###
# SHOW RESULT
###

#cbar = plt.colorbar(allpoints)
#cbar.set_label(r'$\Delta E_\textrm{hull}$\,(meV/atom)', fontsize=18)
plt.tight_layout()
plt.savefig('convex_hull.png', bbox_inches='tight', dpi=300)
plt.show()
